"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./common/utils");
var fetch_1 = __importDefault(require("./interceptor/fetch"));
var wx_request_1 = __importDefault(require("./interceptor/wx-request"));
var xml_http_request_1 = __importDefault(require("./interceptor/xml-http-request"));
var mocker_1 = __importDefault(require("./mocker/mocker"));
var Index = /** @class */ (function () {
    function Index() {
    }
    /**
     * Auto detect request enviroment and setup request mock.
     * @param {string} type
     */
    Index.setup = function () {
        var mocker = new mocker_1.default();
        if (typeof wx !== 'undefined' && typeof wx.request === 'function') {
            wx_request_1.default.setup(mocker);
        }
        if (typeof window !== 'undefined' && typeof window.XMLHttpRequest === 'function') {
            xml_http_request_1.default.setup(mocker);
        }
        if (typeof window !== 'undefined' && typeof window.fetch === 'function') {
            fetch_1.default.setup(mocker);
        }
        // for http.get, https.get, http.request, https.request in node enviroment
        if ((0, utils_1.isNodejs)()) {
            require('./interceptor/node/http-and-https').default.setup(mocker);
        }
        return mocker;
    };
    /**
     * Setup request mock for wx.request.
     * @param {string} type
     */
    Index.setupForWx = function () {
        var mocker = new mocker_1.default();
        wx_request_1.default.setup(mocker);
        return mocker;
    };
    /**
     * Setup request mock for XMLHttpRequest.
     * @param {string} type
     */
    Index.setupForXhr = function () {
        var mocker = new mocker_1.default();
        xml_http_request_1.default.setup(mocker);
        return mocker;
    };
    /**
     * Setup request mock for fetch.
     * @param {string} type
     */
    Index.setupForFetch = function () {
        var mocker = new mocker_1.default();
        fetch_1.default.setup(mocker);
        return mocker;
    };
    /**
     * Setup request mock for node http/https request.
     * @param {string} type
     */
    Index.setupForNode = function () {
        var mocker = new mocker_1.default();
        require('./interceptor/node/http-and-https').default.setup(mocker);
        return mocker;
    };
    /**
     * Enable mock function temporarily.
     */
    Index.enable = function () {
        return mocker_1.default.getInstance().enable();
    };
    /**
     * Disable mock function temporarily.
     */
    Index.disable = function () {
        return mocker_1.default.getInstance().disable();
    };
    /**
     * Enable verbose log.
     */
    Index.enableLog = function () {
        return mocker_1.default.getInstance().enableLog();
    };
    /**
     * Disable verbose log.
     */
    Index.disableLog = function () {
        return mocker_1.default.getInstance().disableLog();
    };
    /**
     * Setup request mock for unit test.
     * @param {string} type
     */
    Index.setupForUnitTest = function (type) {
        if (!(0, utils_1.isNodejs)()) {
            throw new Error('"setupForUnitTest" is only for nodejs envrioment.');
        }
        var mocker = new mocker_1.default();
        if (type === 'wx') {
            wx_request_1.default.setupForUnitTest(mocker);
        }
        if (type === 'xhr') {
            xml_http_request_1.default.setupForUnitTest(mocker);
        }
        if (type === 'fetch') {
            fetch_1.default.setupForUnitTest(mocker);
        }
        if (type === 'node' || type === 'node.http.request') {
            require('./interceptor/node/http-and-https').default.setupForUnitTest(mocker);
        }
        if (type === 'all') {
            wx_request_1.default.setupForUnitTest(mocker);
            xml_http_request_1.default.setupForUnitTest(mocker);
            fetch_1.default.setupForUnitTest(mocker);
            require('./interceptor/node/http-and-https').default.setupForUnitTest(mocker);
        }
        return mocker;
    };
    return Index;
}());
exports.default = Index;
//# sourceMappingURL=index.js.map