"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var http_1 = __importDefault(require("http"));
var net_1 = require("net");
var util_1 = require("util");
var bypass_1 = __importDefault(require("../../common/bypass"));
var utils_1 = require("../../common/utils");
var config_1 = require("../../config");
/**
 * ClientRequest constructor
 * @param {string} url
 * @param {object} options options of http.get, https.get, http.request or https.request method.
 * @param {function} callback callback of http.get, https.get, http.request or https.request method.
 */
function ClientRequest(url, options, callback) {
    var _this = this;
    // http.OutgoingMessage serves as the parent class of http.ClientRequest and http.ServerResponse.
    // It is an abstract of outgoing message from the perspective of the participants of HTTP transaction.
    http_1.default.OutgoingMessage.call(this);
    this.requestBody = Buffer.alloc(0);
    this.url = url;
    this.options = options;
    this.callback = callback;
    this.nativeInstance = null;
    /**
     * Initialize socket & response object
     */
    this.init = function () {
        var _a;
        var _b = [_this.options, _this.callback], options = _b[0], callback = _b[1];
        _this.method = options.method || 'GET';
        _this.path = options.path || '/';
        // The optional callback parameter will be added as
        // a one-time listener for the 'response' event.
        if (callback) {
            _this.once('response', callback);
        }
        // outgoingMessage.headersSent
        if (!_this.headersSent && options.headers) {
            for (var key in options.headers) {
                _this.setHeader(key, options.headers[key]);
            }
        }
        // make an empty socket
        _this.socket = new net_1.Socket();
        _this.connection = _this.socket; // for compatibility
        if (/^https/i.test(_this.url)) {
            // @ts-ignore
            _this.socket.authorized = true;
        }
        if (options.timeout) {
            _this.setTimeout(options.timeout);
            _this.socket.setTimeout(options.timeout);
        }
        process.nextTick(function () {
            var _a;
            _this.emit('socket', _this.socket);
            (_a = _this.socket) === null || _a === void 0 ? void 0 : _a.emit('connect');
        });
        if (((_a = options.headers) === null || _a === void 0 ? void 0 : _a.expect) === '100-continue') {
            _this.emit('continue');
        }
        _this.response = new http_1.default.IncomingMessage(_this.socket);
    };
    this.init();
    /**
     * Set mock item resolver. 'mockItemResolver' will be used in end method.`
     * @param {Promise<MockItem>} mockItemResolver
     */
    this.setMockItemResolver = function (mockItemResolver) {
        _this.mockItemResolver = mockItemResolver;
        return _this;
    };
    this.setOriginalRequestInfo = function (nativeReqestMethod, nativeRequestArgs) {
        _this.nativeReqestMethod = nativeReqestMethod;
        _this.nativeRequestArgs = nativeRequestArgs;
    };
    /**
     * Destroy the request. Optionally emit an 'error' event, and emit a 'close' event.
     * Calling this will cause remaining data in the response to be dropped and the socket to be destroyed.
     */
    this.destroy = function () {
        if (_this.aborted || _this.destroyed)
            return;
        _this.aborted = true;
        _this.destroyed = true;
        var error = new Error();
        error.code = 'aborted';
        _this.response.emit('close', error);
        // socket.destroy()
        _this.emit('abort');
        return _this;
    };
    /**
     * We keep abort method for compatibility.
     * 'abort' has been Deprecated; Use request.destroy() instead.
     */
    this.abort = function () {
        _this.destroy();
        return _this;
    };
    /**
     * Send error event to the request.
     * @param {string} msg
     */
    this.sendError = function (msg) {
        process.nextTick(function () {
            _this.emit('error', new Error(msg));
        });
    };
    /**
     * Sends a chunk of the body. This method can be called multiple times.
     * Simulation: request.write(chunk[, encoding][, callback])
     * @param {string | Buffer} chunk
     * @param {any[]} args
     */
    this.write = function (chunk) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (typeof chunk !== 'string' && !Buffer.isBuffer(chunk)) {
            _this.sendError('The first argument must be of type string or an instance of Buffer.');
            return false;
        }
        var callback = typeof args[1] === 'function' ? args[1] : args[2];
        if (_this.aborted || _this.destroyed) {
            _this.sendError('The request has been aborted.');
        }
        else {
            if (chunk.length) {
                var buf = Buffer.isBuffer(chunk) ? chunk : Buffer.from(chunk);
                _this.requestBody = Buffer.concat([_this.requestBody, buf]);
            }
            // The callback argument is optional and will be called when this
            // chunk of data is flushed, but only if the chunk is non-empty.
            if (chunk.length && typeof callback === 'function') {
                callback();
            }
        }
        setTimeout(function () { return _this.emit('drain'); }, 1);
        return false;
    };
    /**
     * https://nodejs.org/api/http.html#http_request_end_data_encodingcallback
     *
     * Finishes sending the request. If any parts of the body are unsent, it will flush them to the stream.
     * Simulation: request.end([data[, encoding]][, callback])
     * @param {any[]} args
     */
    this.end = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _a = _this.getEndArguments(args), data = _a[0], encoding = _a[1], callback = _a[2];
        // If data is specified, it is equivalent to calling
        // request.write(data, encoding) followed by request.end(callback).
        if (data) {
            _this.write(data, encoding);
            _this.end(callback);
            return _this;
        }
        if (!_this.response.complete) {
            _this.sendResponseResult.apply(_this, __spreadArray([callback], args, false));
            return _this;
        }
        _this.sendEndingEvent(callback);
        return _this;
    };
    /**
     * It awaits mock item resolver & set response result.
     */
    this.sendResponseResult = function (endCallback) {
        var endArgs = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            endArgs[_i - 1] = arguments[_i];
        }
        var now = Date.now();
        _this.mockItemResolver(function (mockItem, mocker) {
            var requestInfo = {
                url: _this.url,
                method: _this.options.method || 'GET',
                query: (0, utils_1.getQuery)(_this.url),
                headers: _this.getRequestHeaders(),
                body: _this.bufferToString(_this.requestBody)
            };
            mockItem.sendBody(requestInfo).then(function (responseBody) {
                if (responseBody instanceof bypass_1.default) {
                    return _this.fallbackToNativeRequest.apply(_this, endArgs);
                }
                var spent = Date.now() - now;
                mocker.sendResponseLog(spent, responseBody, requestInfo, mockItem);
                _this.response.statusCode = mockItem.status;
                _this.response.statusMessage = config_1.HTTPStatusCodes[_this.response.statusCode] || '',
                    _this.response.headers = __assign(__assign({}, mockItem.header), { 'x-powered-by': 'http-request-mock' });
                _this.response.rawHeaders = Object.entries(_this.response.headers).reduce(function (res, item) {
                    return res.concat(item);
                }, []);
                // push: The "chunk" argument must be of type string or an instance of Buffer or Uint8Array.
                if (typeof responseBody === 'string'
                    || (responseBody instanceof Buffer)
                    || (responseBody instanceof ArrayBuffer)
                    || (responseBody instanceof SharedArrayBuffer)
                    || (responseBody instanceof Uint8Array)) {
                    _this.response.push(Buffer.from(responseBody));
                }
                else {
                    _this.response.push(JSON.stringify(responseBody));
                }
                _this.sendEndingEvent(endCallback);
            });
        });
    };
    /**
     * Send completed event.
     */
    this.sendEndingEvent = function (callback) {
        if (typeof callback === 'function') {
            callback();
        }
        _this.finished = true; // We keep the finish property for compatibility.
        _this.emit('finish');
        _this.emit('response', _this.response);
        // The message.complete property will be true if a complete
        // HTTP message has been received and successfully parsed.
        _this.response.push(null);
        _this.response.complete = true;
        return _this;
    };
    this.fallbackToNativeRequest = function () {
        var _a;
        var endArgs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            endArgs[_i] = arguments[_i];
        }
        _this.nativeInstance = _this.nativeReqestMethod.apply(_this, _this.nativeRequestArgs);
        // @ts-ignore
        Object.entries(_this.getHeaders()).forEach(function (entry) {
            if (entry[1] !== null && entry[1] !== undefined) {
                _this.nativeInstance.setHeader(entry[0], entry[1]);
            }
        });
        if (_this.requestBody.length) {
            // @ts-ignore
            _this.nativeInstance.write(_this.requestBody);
        }
        // @ts-ignore
        _this.nativeInstance.on('connect', function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return _this.emit.apply(_this, __spreadArray(['connect'], args, false));
        });
        // @ts-ignore
        _this.nativeInstance.on('finish', function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return _this.emit.apply(_this, __spreadArray(['finish'], args, false));
        });
        // @ts-ignore
        _this.nativeInstance.on('abort', function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return _this.emit.apply(_this, __spreadArray(['abort'], args, false));
        });
        // @ts-ignore
        _this.nativeInstance.on('error', function (error) { return _this.emit('error', error); });
        // @ts-ignore
        _this.nativeInstance.on('information', function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return _this.emit.apply(_this, __spreadArray(['information'], args, false));
        });
        // @ts-ignore
        _this.nativeInstance.on('response', function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return _this.emit.apply(_this, __spreadArray(['response'], args, false));
        });
        // @ts-ignore
        _this.nativeInstance.on('timeout', function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return _this.emit.apply(_this, __spreadArray(['timeout'], args, false));
        });
        // @ts-ignore
        (_a = _this.nativeInstance).end.apply(_a, endArgs);
        return _this.nativeInstance;
    };
    /**
     * https://nodejs.org/api/http.html#http_request_end_data_encodingcallback
     *
     * Get arguments of end method.
     * @param {any[]} args [data[, encoding]][, callback]
     * @returns
     */
    this.getEndArguments = function (args) {
        var data;
        var encoding;
        var callback;
        if (args.length === 3) {
            data = args[0], encoding = args[1], callback = args[2];
        }
        else if (args.length === 2) {
            data = args[0], encoding = args[1];
        }
        else if (args.length === 1) {
            data = typeof args[0] === 'function' ? undefined : args[0];
            callback = typeof args[0] === 'function' ? args[0] : undefined;
        }
        return [data, encoding, callback];
    };
    /**
     * Convert a buffer to a string.
     * @param {Buffer} buffer
     */
    this.bufferToString = function (buffer) {
        var str = buffer.toString('utf8');
        return Buffer.from(str).equals(buffer) ? str : buffer.toString('hex');
    };
    /**
     * Get request headers.
     */
    this.getRequestHeaders = function () {
        return Object.entries(__assign(__assign({}, _this.getHeaders()), _this.options.headers)).reduce(function (res, _a) {
            var key = _a[0], val = _a[1];
            if (val !== undefined && val !== null) {
                res[key.toLowerCase()] = Array.isArray(val)
                    ? val.join('; ')
                    : (val + '');
            }
            return res;
        }, {});
    };
}
// Note: 'class extends' is not work here.
// It'll trigger a default socket connection that we don't expect.
(0, util_1.inherits)(ClientRequest, http_1.default.ClientRequest);
exports.default = ClientRequest;
//# sourceMappingURL=client-request.js.map